////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// App.h
// Application class definition
//
////////////////////////////////////////////////////////////////////////////////

#ifndef APP_H
#define APP_H

#include <vector>

#include "png.h"
#include "SDL.h"

#include "EventHandler.h"
#include "Mandelbrot.h"


class App : public EventHandler
   {
   public:

   App();
   ~App();

   int run();

   private:

   bool init();
   void update();
   void render();
   void cleanup();

   // Overrides of EventHandler:
   void on_event(SDL_Event * event);
   void on_KeyDown(SDLKey sym, SDLMod mod, Uint16 unicode);
   void on_KeyUp(SDLKey sym, SDLMod mod, Uint16 unicode);
   void on_Exit();
   void on_LButtonDown(int mx, int my);
   void on_LButtonUp(int mx, int my);
   void on_MouseMove(int mx, int my, 
		     int relx, int rely, 
		     bool left, bool right, bool middle);
   void on_Resize(int w, int h);
   void on_Expose();


   int mouse_move_width(int mx, int my);
   bool create_new_surface(SDL_Surface * &surface);
   void restore_view();
   void set_caption();
   bool save_image();

   static Uint32 timer_callback(Uint32 interval, void *param);


   bool running;
   bool redraw;

   SDL_Surface * surf_display;
   SDL_Surface * surf_selection;
   SDL_Surface * surf_fractal;

   int selection_x, selection_y, selection_width;
   bool setting_zoom;
   bool can_set_julia_k;
   bool setting_julia_k;

   struct view
      {
      double re_min, im_max, size;
      };
   static std::vector<view *> old_views;

   Fractal * fractal;
   };

#endif
