////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// EventHandler.cpp
// EventHandler class implementation.
//
////////////////////////////////////////////////////////////////////////////////
#include <iostream>

#include "EventHandler.h"


////////////////////////////////////////////////////////////////////////////////
EventHandler::EventHandler()
   {
   }



////////////////////////////////////////////////////////////////////////////////
EventHandler::~EventHandler()
   {
   }



////////////////////////////////////////////////////////////////////////////////
// Note: SDL 1.3 adds an official SDL_WaitEventTimeout function.
#ifndef USE_SDL_WAIEVENT
int EventHandler::wait_event_timeout(SDL_Event * event, Uint32 timeout)
   {
   Uint32 end = SDL_GetTicks() + timeout;
   int i;

   for (;;) 
      {
      SDL_PumpEvents();
      i = SDL_PeepEvents(event, 1, SDL_GETEVENT, SDL_ALLEVENTS);
      switch (i) 
	 {
	 case -1:
	    return 0;
	 case 1:
	    return 1;
	 case 0:
	    if (SDL_GetTicks() >= end)
	       {
	       return 0;
	       }
	    SDL_Delay(10);
	    break;
	 }
      }
   }
#endif


////////////////////////////////////////////////////////////////////////////////
void EventHandler::on_event(SDL_Event * event)
   {
   switch(event->type)
      {
      case SDL_ACTIVEEVENT: 
	 {
	 switch(event->active.state)
	    {	   
	    case SDL_APPMOUSEFOCUS:
	       {
	       if (event->active.gain) on_MouseFocus();
	       else on_MouseBlur();
	       break;
	       }
	    case SDL_APPINPUTFOCUS:
	       {
	       if (event->active.gain) on_InputFocus();
	       else on_InputBlur();
	       break;
	       }
	    case SDL_APPACTIVE:
	       {
	       if (event->active.gain) on_Restore();
	       else on_Minimize();
	       break;
	       }
	    }
	 break;
	 }
      case SDL_KEYDOWN:
	 {
	 on_KeyDown(event->key.keysym.sym, event->key.keysym.mod, 
		    event->key.keysym.unicode);
	 break;
	 }
      case SDL_KEYUP:
	 {
	 on_KeyUp(event->key.keysym.sym, event->key.keysym.mod, 
		    event->key.keysym.unicode);
	 break;
	 }
      case SDL_MOUSEMOTION: 
	 {
	 on_MouseMove(event->motion.x,
		      event->motion.y,
		      event->motion.xrel,
		      event->motion.yrel,
		      (event->motion.state & SDL_BUTTON(SDL_BUTTON_LEFT)) !=0,
		      (event->motion.state & SDL_BUTTON(SDL_BUTTON_RIGHT)) !=0,
		      (event->motion.state & SDL_BUTTON(SDL_BUTTON_MIDDLE))!=0);
	 break;
	 }
      case SDL_MOUSEBUTTONDOWN:
	 {
	 switch (event->button.button)
	    {
	    case SDL_BUTTON_LEFT:
	       {
	       on_LButtonDown(event->button.x, event->button.y);
	       break;
	       }
	    case SDL_BUTTON_RIGHT:
	       {
	       on_RButtonDown(event->button.x, event->button.y);
	       break;
	       }
	    case SDL_BUTTON_MIDDLE:
	       {
	       on_MButtonDown(event->button.x, event->button.y);
	       break;
	       }
	    }
	 break;
	 }
      case SDL_MOUSEBUTTONUP:
	 {
	 switch (event->button.button)
	    {
	    case SDL_BUTTON_LEFT:
	       {
	       on_LButtonUp(event->button.x, event->button.y);
	       break;
	       }
	    case SDL_BUTTON_RIGHT:
	       {
	       on_RButtonUp(event->button.x, event->button.y);
	       break;
	       }
	    case SDL_BUTTON_MIDDLE:
	       {
	       on_MButtonUp(event->button.x, event->button.y);
	       break;
	       }
	    }
	 break;
	 }
      case SDL_JOYAXISMOTION:
	 {
	 on_JoyAxis(event->jaxis.which, event->jaxis.axis, event->jaxis.value);
	 break;
	 }
      case SDL_JOYBALLMOTION:
	 {
	 on_JoyBall(event->jball.which, event->jball.ball, 
		    event->jball.xrel, event->jball.yrel);
	 break;
	 }
      case SDL_JOYHATMOTION:
	 {
	 on_JoyHat(event->jhat.which, event->jhat.hat, event->jhat.value);
	 break;
	 }
      case SDL_JOYBUTTONDOWN:
	 {
	 on_JoyButtonDown(event->jbutton.which, event->jbutton.button);
	 break;
	 }
      case SDL_JOYBUTTONUP:
	 {
	 on_JoyButtonUp(event->jbutton.which, event->jbutton.button);
	 break;
	 }
      case SDL_QUIT:
	 {
	 on_Exit();
	 break;
	 }
      case SDL_SYSWMEVENT:
	 {
	 // Ignore
	 break;
	 }
      case SDL_VIDEORESIZE:
	 {
	 on_Resize(event->resize.w, event->resize.h);
	 break;
	 }
      case SDL_VIDEOEXPOSE:
	 {
	 on_Expose();
	 break;
	 }
      default:
	 {
	 on_User(event->user.type, event->user.code, 
		 event->user.data1, event->user.data2);
	 break;
	 }
      }
   }



////////////////////////////////////////////////////////////////////////////////
void EventHandler::on_InputFocus()
   {
   }


void EventHandler::on_InputBlur()
   {
   }


void EventHandler::on_KeyDown(SDLKey sym, SDLMod mod, Uint16 unicode)
   {
   }


void EventHandler::on_KeyUp(SDLKey sym, SDLMod mod, Uint16 unicode)
   {
   }


void EventHandler::on_MouseFocus()
   {
   }


void EventHandler::on_MouseBlur()
   {
   }

void EventHandler::on_MouseMove(int mx, int my, 
			 int relx, int rely, 
			 bool left, bool right, bool middle)
   {
   }


void EventHandler::on_MouseWheel(bool up, bool down)
   {
   }


void EventHandler::on_LButtonDown(int mx, int my)
   {
   }


void EventHandler::on_LButtonUp(int mx, int my)
   {
   }


void EventHandler::on_RButtonDown(int mx, int my)
   {
   }


void EventHandler::on_RButtonUp(int mx, int my)
   {
   }


void EventHandler::on_MButtonDown(int mx, int my)
   {
   }


void EventHandler::on_MButtonUp(int mx, int my)
   {
   }


void EventHandler::on_JoyAxis(Uint8 ehich, Uint8 axis, Sint16 value)
   {
   }


void EventHandler::on_JoyButtonDown(Uint8 which, Uint8 button)
   {
   }


void EventHandler::on_JoyButtonUp(Uint8 which, Uint8 button)
   {
   }


void EventHandler::on_JoyHat(Uint8 which, Uint8 hat, Uint8 value)
   {
   }


void EventHandler::on_JoyBall(Uint8 which, Uint8 ball, Sint16 xrel, Sint16 yrel)
   {
   }


void EventHandler::on_Minimize()
   {
   }


void EventHandler::on_Restore()
   {
   }


void EventHandler::on_Resize(int w, int h)
   {
   }


void EventHandler::on_Expose()
   {
   }


void EventHandler::on_Exit()
   {
   }


void EventHandler::on_User(Uint8 type, int code, void * data1, void * data2)
   {
   }


